@@ -1,8 +1,10 @@
-from collections import defaultdict
 import numpy as np
-from scipy.spatial.distance import cdist as sp_cdist
 import warnings
 
+from collections import defaultdict
+from scipy.spatial.distance import cdist as sp_cdist
+from typing import Callable
+
 from .base_viewer import BaseViewer
 from ..cooking_machine import BaseDataset
 
@@ -42,7 +44,7 @@ because they didn\'t have frequency after colon sign ":"\
 WARNING_FEWER_THAN_REQUESTED = """\
 Only "{0}" documents available{1}. \
 This is smaller than the requested number of top similar documents "{2}". \
-So output is going to contain all "{0}" documents, but sorted by distance\
+So display is going to contain all "{0}" documents, but sorted by distance\
 """
 
 WARNING_TOO_MANY_REQUESTED = """\
@@ -51,6 +53,57 @@ is bigger than total number of documents in the dataset "{1}"\
 """
 
 
+def prepare_doc_html_with_similarity(
+    document,
+    distance,
+    num_digits: int = 3,
+    num_sentences_in_snippet: int = 4,
+    num_words: int = 15,
+):
+    """
+    Prepares intital document and search results
+    html strings
+
+    Parameters
+    ----------
+    document : Padas.DataFrame row
+        a row that contains columns raw_text
+        and index in string form
+    distance : float between 0 and 1
+        measure of how close found document to the
+        initial inquiry
+    num_digits
+        ammount of digits to visualize as document simmilarity
+    num_sentences_in_snippet
+        how many sentences to use for document snippet
+    num_words
+        number of document words before the line break in
+        the document snippet
+
+    Returns
+    -------
+    doc_html : str
+        an html string with data about document
+        plus additional info for the output clarification
+    """
+    if distance > 0:
+        sim = str(1 - distance)[:2 + num_digits]
+        doc_title = f'<h3>{document.index.values[0]} &emsp; similarity: {sim}</h3>'
+    else:
+        doc_title = f'<h3>Search document: &emsp; {document.index.values[0]}</h3>'
+    get_sentences = document['raw_text'].values[0].split('. ')[:num_sentences_in_snippet]
+    doc_snippet = '. '.join(get_sentences).split(' ')
+    doc_snippet[-1] += '.'
+    doc_snippet = ' '.join([
+        word + '<br />' if i % num_words + 1 == num_words
+        else word for i, word in enumerate(doc_snippet)
+    ])
+    doc_html = f"<h3>{doc_title}</h3>{doc_snippet}<br><br />"
+    if distance == 0:
+        doc_html += '<h2>Search results:</h2>'
+    return doc_html
+
+
 class TopSimilarDocumentsViewer(BaseViewer):
     def __init__(self, model, dataset):
         """Viewer which uses topic model to find documents similar to given one
@@ -124,6 +177,67 @@ class TopSimilarDocumentsViewer(BaseViewer):
 
         return documents_ids, distances
 
+    def view_from_jupyter(
+            self,
+            document_id: str,
+            metric: str or Callable = 'jensenshannon',
+            num_top_similar: int = 5,
+            num_digits: int = 3,
+            keep_similar_by_words: bool = True,
+            display_output: bool = True,
+            give_html: bool = False,):
+        """
+        Method for viewing documents similar to requested one
+        from jupyter notebook. Provides document titles and snippets of
+        first few sentences.
+
+        Parameters
+        ----------
+        document_id
+            ID of the document in `dataset`
+        metric
+            Distance measure which is to be used to measure how documents differ from each other
+            If str -- should be one of 'jensenshannon', 'euclidean', 'cosine', 'correlation' --
+                as in scipy.spatial.distance.cdist
+            If callable -- should map two vectors to numeric value
+        num_top_similar
+            How many top similar documents' IDs to show
+        keep_similar_by_words
+            Whether or not to keep in the output those documents
+            that are similar to the given one by their constituent words and words' frequencies
+        display_output
+            if provide output at the end of method run
+        give_html
+            return html string generated by the method
+
+        Returns
+        -------
+        topic_html
+            html string of the generated output
+        """
+        from IPython.display import display_html
+        from topicnet.cooking_machine.pretty_output import make_notebook_pretty
+
+        make_notebook_pretty()
+        search_ids, search_distances = self.view(
+            document_id=document_id,
+            metric=metric,
+            num_top_similar=num_top_similar,
+            keep_similar_by_words=keep_similar_by_words,
+        )
+
+        topic_html = ''
+        search_ids = [document_id] + search_ids
+        search_distances = [0] + search_distances
+        for doc_id, distance in zip(search_ids, search_distances):
+            document = self._dataset.get_source_document(doc_id)
+            topic_html += prepare_doc_html_with_similarity(document, distance)
+        if display_output:
+            display_html(topic_html, raw=True)
+
+        if give_html:
+            return topic_html
+
     def _view(self,
               document_index,
               metric,
