@@ -309,6 +309,13 @@ class TopTokensViewer(BaseViewer):
 
         self.alpha = alpha
         self._dataset = dataset
+        self._cached_top_tokens = None
+
+    @property
+    def cached_top_tokens(self):
+        if self._cached_top_tokens is None:
+            self._cached_top_tokens = self.view(three_levels=False)
+        return self._cached_top_tokens
 
     def _get_target_values(self, phi):
         """
@@ -414,8 +421,7 @@ class TopTokensViewer(BaseViewer):
 
     def to_html(
             self,
-            topic_top_tokens=None,
-            topic_names: List[str] = None,
+            topic_names: Union[str, List[str]] = None,
             digits: int = 5,
             thresh: float = None,  # Deprecated
             horizontally_stack: bool = True) -> str:
@@ -424,10 +430,6 @@ class TopTokensViewer(BaseViewer):
 
         Parameters
         ----------
-        topic_top_tokens : dict of dicts [Deprecated]
-            Dict where first level keys are topic names
-            Second level keys are modalities
-            Third level keys are tokens with their scores as float values
         topic_names : list of strings
             Initial dictionary keys
         digits : int
@@ -449,14 +451,15 @@ class TopTokensViewer(BaseViewer):
         >>> # or
         >>> HTML(viewer.to_html())
         """
-        if topic_top_tokens is not None:  # TODO: remove topic_top_tokens some day
-            warnings.warn(
-                'Don\'t specify `topic_top_tokens` in `to_html()`',
-                DeprecationWarning
-            )
-
-            if topic_names is not None:
-                topic_names = [t for t in topic_names if t in topic_top_tokens.keys()]
+        if topic_names is not None:
+            if isinstance(topic_names, str):
+                topic_names = [topic_names]
+            num_topics_requested = len(topic_names)
+            topic_names = [t for t in topic_names if t in self._model.topic_names]
+            if len(topic_names) < num_topics_requested:
+                warnings.warn(
+                    'Some of the requested topics are absent from the model',
+                )
 
         if thresh is not None:  # TODO: remove thresh some day
             warnings.warn(
@@ -485,14 +488,13 @@ class TopTokensViewer(BaseViewer):
         return convert_df_to_html(df)
 
     def to_df(self, topic_names: Iterator[str] = None, digits: int = 5) -> pd.DataFrame:
-        topic_top_tokens = self.view(three_levels=False)
+        topic_top_tokens = self.cached_top_tokens
 
         if topic_names is not None:
             topic_top_tokens = {
                 topic: tokens for topic, tokens in topic_top_tokens.items()
                 if topic in topic_names
             }
-
         if not isinstance(digits, int):
             warnings.warn(
                 f'Need "int" digits. '
@@ -507,9 +509,7 @@ class TopTokensViewer(BaseViewer):
     def _to_df(
             topic_top_tokens: Dict[str, Dict[Tuple[str, str], float]],
             digits: int) -> pd.DataFrame:
-
         df = pd.DataFrame.from_dict(topic_top_tokens).round(digits)
-
         df.index = pd.MultiIndex.from_tuples(
             df.index,
             names=['modality', 'token']  # TODO: names should be the same as in TopicModel's Phi?
@@ -517,4 +517,81 @@ class TopTokensViewer(BaseViewer):
 
         df.fillna(0.0, inplace=True)
 
+        # Due to some problems with pandas following crunch is applied:
+        if len(df.columns) == 1:
+            col_to_sort_by = df.columns.values[0]
+            return (df.set_index(col_to_sort_by, append=True)
+                    .sort_index(level=[0, 2], ascending=[True, False])
+                    .reset_index(col_to_sort_by))
+
         return df
+
+    def view_from_jupyter(
+            self,
+            topic_names: Union[str, List[str]] = None,
+            digits: int = 5,
+            horizontally_stack: bool = True,
+            display_output: bool = True,
+            give_html: bool = False,
+    ):
+        """
+        TopTokensViewer method recommended for use
+        from jupyter notebooks
+
+        Parameters
+        ----------
+        topic_names
+            topics requested for viewing
+        digits
+            Number of digits to round each probability to
+        horizontally_stack
+            if True, then tokens for each modality will be stacked horizontally
+            (instead of being a single long multi-line DataFrame)
+        display_output
+            request for function to output the information
+            together with iterable output intended to be used
+            as user defined output
+        give_html
+            return html string generated by the method
+
+        Returns
+        -------
+        topic_html_strings: list of strings in HTML format
+
+        Examples
+        --------
+        >>> # model training here
+        >>> # ...
+        >>> viewer = TopTokensViewer(model)
+        >>> information = viewer.view_from_jupyter()
+        >>> # or
+        >>> information = viewer.view_from_jupyter(output=False)
+        """
+        from IPython.core.display import display_html
+        from topicnet.cooking_machine.pretty_output import make_notebook_pretty
+
+        make_notebook_pretty()
+        if isinstance(topic_names, list):
+            pass
+        elif isinstance(topic_names, str):
+            topic_names = [topic_names]
+        elif topic_names is None:
+            topic_names = self._model.topic_names
+        else:
+            raise TypeError(f'Invalid type `topic_names` type: "{type(topic_names)}"')
+
+        topic_html_strings = []
+
+        for topic in topic_names:
+            topic_html = self.to_html(
+                topic_names=topic,
+                digits=digits,
+                horizontally_stack=horizontally_stack,
+            )
+
+            if display_output:
+                display_html(topic_html, raw=True)
+
+            topic_html_strings.append(topic_html)
+        if give_html:
+            return topic_html_strings
